@@ -1,7 +1,7 @@
 package App::AcmeCpanlists;
 
-our $DATE = '2015-10-22'; # DATE
-our $VERSION = '0.03'; # VERSION
+our $DATE = '2015-10-23'; # DATE
+our $VERSION = '0.04'; # VERSION
 
 use 5.010001;
 use strict;
@@ -24,6 +24,27 @@ sub _complete_module {
     );
 }
 
+sub _complete_summary_or_id {
+    require Complete::Util;
+    my %args = @_;
+
+    my $res = list_lists(detail=>1);
+    my $array;
+    if ($res->[0] == 200) {
+        for (@{ $res->[2] }) {
+            push @$array, $_->{id} if $_->{id};
+            push @$array, $_->{summary};
+        }
+    } else {
+        $array = [];
+    }
+
+    Complete::Util::complete_array_elem(
+        %args,
+        array => $array,
+    );
+}
+
 my %rels_filtering = (
     choose_one => [qw/mentions_author mentions_module/],
 );
@@ -58,6 +79,15 @@ my %arg_detail = (
     },
 );
 
+my %arg_query = (
+    query => {
+        schema => 'str*',
+        req => 1,
+        pos => 0,
+        completion => \&_complete_summary_or_id,
+    },
+);
+
 $SPEC{list_mods} = {
     v => 1.1,
     summary => 'List all installed Acme::CPANLists modules',
@@ -117,9 +147,9 @@ sub list_lists {
 
     my @cols;
     if ($detail) {
-        @cols = (qw/name type summary num_entries mentioned_authors_or_modules/);
+        @cols = (qw/id type summary num_entries mentioned_authors_or_modules/);
     } else {
-        @cols = (qw/name/);
+        @cols = (qw/summary/);
     }
 
     my @rows;
@@ -129,14 +159,17 @@ sub list_lists {
                 my $entries = $l->{entries} // [];
                 my $rec = {
                     type => 'author',
+                    id => $l->{id},
                     module => $mod,
                     summary => $l->{summary},
                     num_entries => ~~@$entries,
                 };
 
                 my %mentioned;
-                for (@$entries) {
-                    $mentioned{$_->{author}}++;
+                for my $ent (@$entries) {
+                    $mentioned{$ent->{author}}++;
+                    $mentioned{$_}++ for @{ $ent->{related_authors} // [] };
+                    $mentioned{$_}++ for @{ $ent->{alternate_authors} // [] };
                 }
                 $rec->{mentioned_authors_or_modules} = join(", ", sort keys %mentioned);
 
@@ -156,15 +189,17 @@ sub list_lists {
                 my $entries = $l->{entries} // [];
                 my $rec = {
                     type => 'module',
+                    id => $l->{id},
                     module => $mod,
                     summary => $l->{summary},
                     num_entries => ~~@$entries,
                 };
 
                 my %mentioned;
-                for (@$entries) {
-                    $mentioned{$_->{module}}++;
-                    $mentioned{$_->{alternate_module}}++ if defined($_->{alternate_module});
+                for my $ent (@$entries) {
+                    $mentioned{$ent->{module}}++;
+                    $mentioned{$_}++ for @{ $ent->{related_modules} // [] };
+                    $mentioned{$_}++ for @{ $ent->{alternate_modules} // [] };
                 }
                 $rec->{mentioned_authors_or_modules} = join(", ", sort keys %mentioned);
 
@@ -186,21 +221,13 @@ sub list_lists {
 
 $SPEC{get_list} = {
     v => 1.1,
-    summary => 'Get a CPAN list',
+    summary => 'Get a CPAN list as raw data',
     args_rels => {
         %rels_filtering,
     },
     args => {
         %args_filtering,
-        %arg_detail,
-        all => {
-            schema => 'bool',
-        },
-        query => {
-            schema => 'str*',
-            req => 1,
-            pos => 0,
-        },
+        %arg_query,
     },
 };
 sub get_list {
@@ -216,18 +243,60 @@ sub get_list {
     );
 
     my @rows;
+    my $type;
     for my $row (@{ $res->[2] }) {
-        if ($row->{summary} eq $args{query}) {
+        if (index(lc($row->{summary}), lc($args{query})) >= 0 ||
+                (defined($row->{id}) &&
+                         index(lc($row->{id}), lc($args{query})) >= 0)) {
             my $rec = $row->{_ref};
-            if ($args{all}) {
-                push @rows, $rec;
-            } else {
-                return [200, "OK", $rec];
-            }
+            $type = $row->{type};
+            push @rows, $rec;
         }
     }
 
-    [200, "OK", \@rows];
+    if (!@rows) {
+        return [404, "No such list"];
+    } elsif (@rows > 1) {
+        return [300, "Multiple lists found (".~~@rows."), please specify"];
+    } else {
+        [200, "OK", $rows[0], {'func.type'=>$type}];
+    }
+}
+
+$SPEC{view_list} = {
+    v => 1.1,
+    summary => 'View a CPAN list as rendered POD',
+    args_rels => {
+        %rels_filtering,
+    },
+    args => {
+        %args_filtering,
+        %arg_query,
+    },
+};
+sub view_list {
+    require Pod::From::Acme::CPANLists;
+    no strict 'refs';
+
+    my %args = @_;
+
+    my $res = get_list(%args);
+    return $res unless $res->[0] == 200;
+
+    my %podargs;
+    if ($res->[3]{'func.type'} eq 'author') {
+        $podargs{author_lists} = [$res->[2]];
+        $podargs{module_lists} = [];
+    } else {
+        $podargs{author_lists} = [];
+        $podargs{module_lists} = [$res->[2]];
+    }
+    my $podres = Pod::From::Acme::CPANLists::gen_pod_from_acme_cpanlists(
+        %podargs);
+
+    [200, "OK", $podres, {
+        "cmdline.page_result"=>1,
+        "cmdline.pager"=>"pod2man | man -l -"}];
 }
 
 1;
@@ -245,9 +314,9 @@ App::AcmeCpanlists - The Acme::CPANLists CLI (backend module)
 
 =head1 VERSION
 
-This document describes version 0.03 of App::AcmeCpanlists (from Perl distribution App-AcmeCpanlists), released on 2015-10-22.
+This document describes version 0.04 of App::AcmeCpanlists (from Perl distribution App-AcmeCpanlists), released on 2015-10-23.
 
-=head1 SEE ALSO
+=head1 SYNOPSIS
 
 Use the included script L<acme-cpanlists>.
 
@@ -256,7 +325,7 @@ Use the included script L<acme-cpanlists>.
 
 =head2 get_list(%args) -> [status, msg, result, meta]
 
-Get a CPAN list.
+Get a CPAN list as raw data.
 
 This function is not exportable.
 
@@ -264,10 +333,6 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<all> => I<bool>
-
-=item * B<detail> => I<bool>
-
 =item * B<mentions_author> => I<str>
 
 =item * B<mentions_module> => I<str>
@@ -345,6 +410,40 @@ that contains extra information.
 
 Return value:  (any)
 
+
+=head2 view_list(%args) -> [status, msg, result, meta]
+
+View a CPAN list as rendered POD.
+
+This function is not exportable.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<mentions_author> => I<str>
+
+=item * B<mentions_module> => I<str>
+
+=item * B<module> => I<str>
+
+=item * B<query>* => I<str>
+
+=item * B<type> => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-AcmeCpanlists>.
